/***********************************************************/
/* CALLING EXAMPLE:                                        */
/* >> N = 50;                                              */
/* >> T = 10;                                              */
/* >> t = [0:T/N:T];                                       */
/* >> u0 = zeros(1,N+2);                                   */
/* >> x0 = [0 0 1]';                                       */
/* >> fixed = [1 1 0]';                                    */
/* >> x0min = [0 0 .1]';                                   */
/* >> x0max = [0 0 10]';                                   */
/* >> X0 = [x0 fixed x0min x0max];                         */
/* >> [u,x,f]=riots(X0,u0,t,-2,1,[],100,2);                */
/***********************************************************/
/* PROBLEM DESCRIPTION:

   Minimize T :

   x1_dot = x2  ;   x1(0) = 0 ;  x1(T) = 300;
   x2_dot = u   ;   x2(0) = 0 ;  x2(T) = 0;
   
   u member [ -2 , 1 ]
   
   Optimal solution: T = 30 
                              |  0 <= t < 20   |   20 <= t <= 30  |
                              +----------------+------------------+
                           u  |       1        |        -2        |
			      +----------------+------------------+
                           x1 |     1/2 t^2    | -t^2 + 60t - 600 |
			      +----------------+------------------+
                           x2 |       t        |      60 - 2t     |
			      +----------------+------------------+
*/

#define NSTATES 3
#define NINPUTS 1
#define LEEC  2            /* Linear endpoint equality constraints.        */
#define FNUM neq[4]

#define TAU x[2]

void activate(message)
     char **message;
{
  *message = "Bang";
} 

void init(neq,params)
     int neq[];
     double *params;
{
  neq[0] = NSTATES;
  neq[1] = NINPUTS;
  neq[11] = LEEC;
}

void h(neq,t,x,u,xdot)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS],xdot[NSTATES];
{
  xdot[0] = TAU*x[1]; 
  xdot[1] = TAU*u[0]; 
  xdot[2] = 0.0;         /* Augmented state tau --- represents final time. */
}

void Dh(neq,t,x,u,h_x,h_u)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS];
     double h_x[NSTATES][NSTATES],h_u[NSTATES][NINPUTS];
{
  h_x[0][1] = TAU;
  h_x[0][2] = x[1]; 
  h_x[1][2] = u[0];

  h_u[1][0] = TAU;
}

double l(neq,t,x,u)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS];
{
  neq[3] = -1;
  return 0.0;
}

double Dl(neq,t,x,u,l_x,l_u)
     int neq[];
     double *t,x[NSTATES],u[NINPUTS],l_x[NSTATES],l_u[NINPUTS];
{
  return 0.0;
}

double g(neq,t,x0,xf)
     int neq[];
     double *t,x0[NSTATES],xf[NSTATES];
{
  switch (FNUM) {
  case 1:
    return x0[2];
    break;
  case 2: 
    return xf[0]/300.0 - 1;
    break;
  case 3:
    return xf[1];
    break;
  }
}

double Dg(neq,t,x0,xf,J_x0,J_xf)
     int neq[]; 
     double *t,x0[NSTATES],xf[NSTATES],J_x0[NSTATES],J_xf[NSTATES];
{
  switch (FNUM) {
  case 1:
    J_x0[2] = 1.0;
    break;
  case 2: 
    J_xf[0] = 1.0/300.0;
    break;
  case 3:
    J_xf[1] = 1.0;
    break;
  }
  return 0.0;
}
