%function gamma = gestimate(f0,x0,u,dz,grad,Y,ialg,I_i,free_x0)
function gamma = gestimat(f0,x0,u,dz,grad,Y,ialg,I_i,free_x0)

%  Function to estimate gamma for a first order descent method. 

%  The global variable Minv is set by pdmin.m.
%
%  dz/gamma is the minimum of a parabolic fit to f0 at [u x0].

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

global USER_FUNCTION_NAME Minv User_function_exist

ninputs  = size(u,1); 
length_u = size(u,2);
ncontrols = ninputs*length_u;
n_free_x0 = length(free_x0);

du = reshape(dz(1:ncontrols),ninputs,length_u);
dx0 = zeros(size(x0));
dx0(free_x0) = dz(ncontrols+1:ncontrols+n_free_x0);
if User_function_exist
  f1 = feval(USER_FUNCTION_NAME,(x0+dx0),(u+du)*Minv,Y,ialg,1);
else
  f1=simulate(1,(x0+dx0),(u+du)*Minv,Y,ialg,1);
end
gamma = 2*abs((f1 - f0 - grad(I_i)'*dz(I_i))/(dz(I_i)'*dz(I_i)));
