%RIOTSdem5	Control error estimation.

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

clear all
clc,clf,home,echo on,hold off

% We will use the Rayleigh optimal control problem for this demonstration.
%
% >!copy rayleigh.mex simulate.mex 
%
pause                              % TOUCH ANY KEY TO CONTINUE.1
clear simulate
!copy rayleigh.mexw64 simulate.mexw64


% There are two programs in the RIOTS package that are used for
% computing various error estimates.  These are est_errors() and 
% control_error().  The output of est_errors() for solutions based on
% two different meshes are required for control_error() to estimate
% the difference between the control solution obtained numerically and
% the (an) actual optimal control.

pause                              % TOUCH ANY KEY TO CONTINUE.2

% We will use riots() to solve the Rayleigh problem with the endpoint
% constraint  x1(2.5) = 0 , first on an initial uniform mesh of  N=50
% intervals and then again on a doubled mesh.  We'll use linear splines
% and the second-order Runge-Kutta integration routine for the first
% solution.  For the second solution we'll use quadratic splines with
% the fourth-order Runge-Kutta integration routine.

clear simulate
global FLAGS
FLAGS = 1;
N  = 50;
x0 = [-5;-5];
u0 = zeros(1,N+2-1);
t  = [0:2.5/N:2.5];

pause                              % TOUCH ANY KEY TO CONTINUE3.

% We will make our call to riots() as follos:
%
% [u1,x1,f1,g1,lambda1] = riots(x0,u0,t,[],[],[],100,2);
% 
% The outputs here include, in addition to the control and trajectory
% solutions, the value of the objective, the value of the constraints
% and the Lagrange multipliers associated with the constraints.  We will
% need the Lagragne multipliers to make our estimate of the control
% solution error.

pause                              % TOUCH ANY KEY TO CONTINUE.4

[u1,x1,f1,g1,lambda1] = riots(x0,u0,t,[],[],[],100,2);

sp_plot(t,u1)
xlabel('Time'),ylabel('Optimal Control')
title('Linear spline solution for the constrained Rayleigh problem')

pause                              % TOUCH ANY KEY TO CONTINUE.5

% Now let's double the mesh and resolve.  We'll use distribute() to 
% double the mesh (strategy 3) since it will interpolate the control
% solution we just obtained onto the new mesh.  Also, since we would
% like to use quadratic splines this time, we can tell distribute()
% to give us a quadratic spline.

[new_t,new_u] = distribu(t,u1,x1,4,[],[],3,[],3);

pause                              % TOUCH ANY KEY TO CONTINUE.6

% In this call to distribute, the inputs 4, 3, and 3 indicate,
% respectively, that the next optimization is to be performed with RK4,
% the mesh refinement strategy is number 3 (just double the mesh) and
% that we want a quadratic spline interpolation to the old control.

% [u2,x2,f2,g2,lambda2] = riots(x0,new_u,new_t,[],[],[],100,4);

pause                              % TOUCH ANY KEY TO CONTINUE.7
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%ERROR
[u2,x2,f2,g2,lambda2] = riots(x0,new_u,new_t,[],[],[],100,4);

sp_plot(new_t,u2)
xlabel('Time'),ylabel('Optimal Control')
title('Quadratic spline solution for the constrained Rayleigh problem')

pause                              % TOUCH ANY KEY TO CONTINUE.8

% Now we have two solutions, (t1,u1,x1) and (t2,u2,x2)---let's do some
% error estimation.  First will use est_errors() to provide us with
% the data we need to give to control_error().  The inputs to
% est_errors() are: the current solution for the initial conditions,
% the current solution for the control, the mesh sequence, the optimal
% duration scale factor (we use 1 here since this is a fixed final
% time problem), the integration routine used to obtain the solution,
% and the Lagrange multlipliers estimates obtained for that solution.

[int_error1,norm_gLa1] = est_erro(x0,u1,t,1,2,lambda1);
[int_error2,norm_gLa2] = est_erro(x0,u2,new_t,1,4,lambda2);

pause                              % TOUCH ANY KEY TO CONTINUE.9

% The first output of est_errors() is an estimate of the global
% integration error.  The second output is the L_2 norm of the
% augmented Lagrangian for the problem being solved evaluated at
% (x0,u,t).  This norm is computed by first re-simulating the system
% with the variable stepsize integration routine.  The values of the
% objective, constraints and trajectories from this simulation are also
% available as outputs from est_errors().

pause                              % TOUCH ANY KEY TO CONTINUE.10

% Now we'll compute and estimate of the control error for both the 
% first and second solution.  Presumably, the second solution will
% have a smaller error.  (The last input indicates that this is a fixed
% final time problem, Tf = 1).

error1 = ctrl_err(x0,u2,new_t,norm_gLa2,x0,u1,t,norm_gLa1,1)
error2 = ctrl_err(x0,u1,t,norm_gLa1,x0,u2,new_t,norm_gLa2,1)

pause                              % TOUCH ANY KEY TO CONTINUE.11

% If there had been free initial conditions, we would have specified
% x1(:,1)  and  x2(:,1)  in place of x0  and  Tf  in place of  1 .
%
% The meaning of this error estimate depends on the units of the
% control.  It also depends on the time interval.  The bigger the time
% interval, the larger the L2 inner-product, and hence the error
% estimate, is for a given control profile.

pause                              % TOUCH ANY KEY TO CONTINUE.12
