%RIOTSdem6	Warm starts.

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

clear all
clc,clf,home,echo on,hold off

% We will again use the Rayleigh optimal control problem for
% this demonstration.
% >!copy rayleigh.mex simulate.mex 
%
pause                              % TOUCH ANY KEY TO CONTINUE.
clear simulate
!copy rayleigh.mexw64 simulate.mexw64


pause                              % TOUCH ANY KEY TO CONTINUE.

% The warm start facility applies only to riots() linked with NPSOL.
% There are two versions of warm starts.  The first lets you call
% riots() so that it continues to optimize from where it last left
% off.  To do this, riots() uses the Lagrange multipliers and its
% estimate of the Hessian of the Lagrangian it saved (if told to do
% so) from the previous run.

pause                              % TOUCH ANY KEY TO CONTINUE.

% To demonstrate this type of warm starting, let's allow riots()
% 10 iterations for the endpoint constrained Rayleigh problem.  We
% will then call riots() again with a warm start to allow it to
% compute the solution to the default tolerance level.

clear simulate
global FLAGS
FLAGS = 1;
N  = 50;
x0 = [-5;-5];
u0 = zeros(1,N+2-1);
t  = [0:2.5/N:2.5];

pause                              % TOUCH ANY KEY TO CONTINUE.

% Our call to riots() will be:
%
% [u,x] = riots(x0,u0,t,[],[],[],10,2,[],0,0,-1);
% 
% The last four inputs specify: default tolerance for the convergence
% tests, no automatic function scaling, output display off, and save
% Lagrange multipliers and Hessian for a possible warm-start of
% riots() later on.

pause                              % TOUCH ANY KEY TO CONTINUE.

[u,x] = riots(x0,u0,t,[],[],[],10,2,[],0,0,-1);

pause                              % TOUCH ANY KEY TO CONTINUE.

% Now we'll perform a warm start from where riots() left off using:
% 
% [u,x] = riots(x0,u,t,[],[],[],100,2,[],0,0,1);
% 
% We're allowing a maximum of 100 iterations this time and starting
% from u rather than u0 .

pause                              % TOUCH ANY KEY TO CONTINUE.

[u,x] = riots(x0,u,t,[],[],[],100,2,[],0,0,1);

pause                              % TOUCH ANY KEY TO CONTINUE.

% The total number of iterations performed by riots() in these two
% calls is 24.  This is exactly the same number of iterations it would
% have taken to find a solution in a single call to riots() had we not
% specified a maximum of ten iterations.

sp_plot(t,u)
xlabel('Time'),ylabel('Optimal Control')
title('Solution using linear splines on a uniform mesh.')

pause                              % TOUCH ANY KEY TO CONTINUE.
clc,home

% There is a more general warm starting facility availabe with riots()
% linked with NPSOL.  The previous type of warm start causes riots()
% to continue from where it left off.  The second type of warm start
% allows riots() to perform a warm start with a new mesh and/or a new
% order for the control spline.  
% 
% For this type of warm start there is no need to inform riots() on
% the previous call that you are planning to make a warm start on the
% next call.  However, the Lagrange multpiliers will be needed for the
% next call.

pause                              % TOUCH ANY KEY TO CONTINUE.

% This time we make the following call to riots():
% 
% [u,x,f,g,lambda] = riots(x0,u0,t,[],[],[],100,2,[],0,0);
% 
% where we no longer tell riots() to store the multpliers and Hessian,
% and we obtain at the output the Lagrange multiplier estimates.

pause                              % TOUCH ANY KEY TO CONTINUE.

[u,x,f,g,lambda] = riots(x0,u0,t,[],[],[],100,2,[],0,0);

pause                              % TOUCH ANY KEY TO CONTINUE.

% Now, we'll use this solution to redistribute the mesh using the 
% following call:
%
% [new_t,new_u,new_lambda] = distribute(t,u,x,4,lambda,0,2,[],3);
%
% In this call to distribute(), the inputs mean:
% (t,u,x) --- the current solution
%       4 --- RK4 will be used in the next call to riots()
%  lambda --- estimate of the multiplier estimates
%       0 --- There are no free initial conditions in this problem
%       2 --- Use mesh refinement strategy 2
%      [] --- No specification for the required reduction in 
%             integration error.
%       3 --- Make the interpolated control a third order spline.
% 
% The outputs are: the new mesh, the new, third order spline which
% interpolates the current control solution on the new mesh, and the
% lagrange multipliers extended to the new mesh.

pause                              % TOUCH ANY KEY TO CONTINUE.

[new_t,new_u,new_lambda] = distribu(t,u,x,4,lambda,0,2,[],3);

pause                              % TOUCH ANY KEY TO CONTINUE.

% We can now perform a warm start from  (new_t,x0,new_u,new_lambda)
% by calling riots() with:
%
% [u2,x2,f2] = riots(x0,new_u,new_t,[],[],[],100,4,TOL,0,0,new_lambda);
% 
% where TOL = [1e-8,1e-6,0,1e4] is a fairly tight set of convergence
% tolerances.
% 
% When we make this call, riots() will first perform a numerical
% estimate of the Hessian of the Lagrangian using finite differences.
% Some expensive linear algebra is performed on this estimate of the
% Hessian to ensure its positive definitness.
% 
% This is a slow process and the overall approach is not efficient.
% However, this process may be very efficient for future versions
% RIOTS designed for large-scale problems.

pause                              % TOUCH ANY KEY TO CONTINUE.888

TOL = [1e-8,1e-6,0,1e4];
[u2,x2,f2] = riots(x0,new_u,new_t,[],[],[],100,4,TOL,0,0,new_lambda);

pause                              % TOUCH ANY KEY TO CONTINUE.

% This second call to riots() only required 3 iterations to find the
% new solution.  Had we called riots() without the warm start, but
% still using the current solution from the previous call to riots(),
% it would have taken 11 iterations.

pause                              % TOUCH ANY KEY TO CONTINUE.

tau = [0:.01:1];                   % Only plot first portion of solution.

subplot(1,2,1)
val1 = sp_plot(t,u,tau);
plot(tau,val1)
axis([0 1 0 7])
xlabel('First unit of time'),ylabel('Optimal Control')
title('Linear spline solution on old mesh')
 
subplot(1,2,2)
val2 = sp_plot(new_t,u2,tau);
plot(tau,val2)
axis([0 1 0 7])
xlabel('First unit of time'),ylabel('Optimal Control')
title('Quadratic spline solution on new mesh')

pause                              % TOUCH ANY KEY TO CONTINUE.
