% RIOTSdem7	Solving a multi-input control problem making use of numerical difference.
%
% Copyright (c) 2003 by Jinsong Liang
% All right reserved.  
%
% Problem 5.2.20, P233, Dynamic Optimization, Arthur E. Bryson, Addison-Wesley, 1999

clear all
clc,clf,home,echo on,hold off

% We will use the Boeing optimal control problem for
% this demonstration.
% >!copy boeing.mex simulate.mex 
%

pause                              % TOUCH ANY KEY TO CONTINUE.
clear simulate
!copy boeing.mexw64 simulate.mexw64

% The file boeing.c implements the Boeing optimal control problem.  
% In this problem, there are six state variables, two control inputs, and
% six endpoint constraints.
% We will solve the problem using third order splines, fourth order Runge-Kutta 
% integration  routine, and numerical finite difference. In many cases, the 
% gradient of functions is unavailable or very hard to come by.

global FLAGS;
FLAGS = 1;
clear simulate
N  = 20;
x0 = [0; 0; 0; 0; 0; 0];
t  = [0:10/N:10];
u0 = zeros(2,N+3-1);               

% u0 = zeros(2,N+3-1) means there are two inputs and third order spline will be used. 

pause							   % TOUCH ANY KEY TO CONTINUE

%[u,x,f] = riots(x0,u0,t,[],[],[],[20, 0, 1], 4);

% The argument [20, 0, 1] to riots() means the meximum number of iterationa allowed
% is 20, no penalty on the piecewise derivative variations of control is added to 
% the objective function, and the gradient of for all functions will be computed by
% finite difference approximations, in which case Dh, Dg, and Dl will not be called.

pause                              % TOUCH ANY KEY TO CONTINUE.

[u,x,f] = riots(x0,u0,t,[],[],[],[20, 0, 1], 4);

% Let's take a look at the solution.
pause                              % TOUCH ANY KEY TO CONTINUE.

sp_plot(t,u);
xlabel('Time')
ylabel('Optimal control')
title('Optimal control vs. Time')

pause                              % TOUCH ANY KEY TO CONTINUE.

plot(t, x)
xlabel('Time')
ylabel('States')
title('States vs. Time')

% Compared with the original plots in page 235, Dynamic Optimization, Authur E. Bryson,
% Addison-Wesley, 1999, there is almost no difference, alghouth the discretization level
% is only 20 (N = 20). This is attibuted to the third order spline and fourth order 
% Funge-Kutta integration.

pause                              % TOUCH ANY KEY TO CONTINUE.

