function val = sp_plot(t,u,tau)

%      Produces a plot of the spline with coefficients 'u'.
%      The order of the spline is presummed to equal 
%      length(u) - length(t) + 2.  If tau is specified, u is
%      plotted at the times  tau .  Otherwise, u is plotted
%      at 100 evenly spaced points.
% 
% The knots sequences is given by augknt(t,order) and the output 
% val = [t;uval]  where  t  are the data points and  uval are the 
% data values if  tau  is not specified.  If  tau  is given, the output
% is just val = uval.

%
% Copyright (c) 1996 by Adam L. Schwartz
% All right reserved.  
%

if nargin < 2,
  disp('Usage: val = sp_plot(t,u,{tau})')
  return
end

N = length(t) - 1;
order = size(u,2) - N + 1;

sp = spmak(augknt(t,order),u);

if nargin == 2
  % Contruct a mesh with 100 points with the same relative spacing
  % as  t .
  npoints = length(t);
  tt = [t(1):(t(npoints)-t(1))/(npoints-1):t(npoints)];
  tau = [t(1):(t(npoints)-t(1))/100:t(npoints)];
  sp_t = spmak(augknt(tt,2),t);
  uniform_t = fnval(sp_t,tau);
  % Make sure arithmetic rounding doesn't screw up endpoints.
  uniform_t(1) = t(1); uniform_t(101) = t(N+1);

  uval = spval(sp,uniform_t);
  plot(uniform_t,uval);
  if nargout > 0, val = [uniform_t;uval]; end
else
  uval = spval(sp,tau);
  val = uval;
end
